/* Borland TDS (Turbo Debugger Symbols) definition.

   Contributed by Troy Rollo - http://www.troy.rollo.com/
   Copyright 2000 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* TDS is the debugging format used by the Borland compilers. We will
   deal with TDS versions 9 and 10. Version 9 was introduced in Borland C++
   4.0, and version 10 is the version used by Borland C++ version 5.5, which
   is the version used for the free command line compiler.

   TDS is almost identical to CodeView debugging information - in fact
   Borland's document describing the format often makes reference to
   CodeView instead of Turbo Debugger. The most notable differences
   are a different signature (NB instead of FB) and that the CodeView
   format does not byte align everything the way Borland TDS does.
   If somebody is interested in implementing CodeView debugging, they
   can use TDS debugging as a good starting point. They will also need
   to get a copy of "cvexefmt.h" which is in the Microsoft Platform
   SDK samples.
 */

/* The Borland documentation for TDS uses "DB", "DW" and "DD" - from the
   MASM pseudo ops for "define word" and "define double word".
   We'll use tds_db, tds_dw and tds_dd to reduce the chance of a namespace
   conflict. */

typedef unsigned char	tds_db;
typedef	unsigned short	tds_dw;
typedef	unsigned int	tds_dd;

/* This file is intended to be able to be build on Borland C++,
   Microsoft C, and GNU C - each of which has a different method
   of byte aligning structures.

   For Borland C++, use #pragma -a1, which can be done as a blanket
   thing for the entire file.

   For GNU C and Microsoft C, we define a macro ALIGNED1 to be wrapped
   around data elements that are not on their natural alignment. */

#ifdef ALIGNED1
#undef ALIGNED1
#endif

#ifdef __BORLANDC__

#pragma -a1

#define ALIGNED1(x) x

#elif defined(__GNUC__)

#define ALIGNED1(x) x __attribute__ ((packed))

#elif defined(_MSC)

#define ALIGNED1(x) __unaligned x

#else

#error "Don't know how to align a member on a 1 byte boundary."

#endif

/* The file header -

   Appears at the start of the debug info, in which case tfh_offset
   is the offset from the start of the debug info to the start of
   the subsection directory.

   If the TDS information is in an executable, the file header
   also appears at the end of the file, with tfh_offset being the
   offset from the end of the file to the start of the debug info. */

typedef struct
{
  struct
  {
    char	tfh_fb[2];		/* "FB"					*/
    char	tfh_version[2];		/* Version number - two hex digits	*/
  } tfh_signature;
  union
  {
    tds_dd	tfh_subsection_dir;	/* Offset to subsection directory	*/
    tds_dd	tfh_start_of_debug_info;/* Offset to start of debug info	*/
  } tfh_offset;
}
tds_file_header;

#define	TDS_VALID_HEADER(h)	(h.tfh_signature.tfh_fb[0] == 'F' && \
	 h.tfh_signature.tfh_fb[1] == 'B')

/* The first subsection directory header is pointed to by the file header at the start
   of the debugging information. If an incremental linker has been used, there can be
   a chain of directory headers. */
typedef struct
{
  tds_dw	tdh_header_size;		/* The size of this structure		*/
  tds_dw	tdh_dirent_size;		/* The size of a tds_directory_entry	*/
  tds_dd	tdh_total_entries;		/* The number of entries		*/
  tds_dd	tdh_next_directory;		/* Offset to the next directory		*/
  tds_dd	tdh_flags;			/* Always zero				*/
}
tds_directory_header;

/* The directory entries appear immediately after the directory header */
typedef struct
{
  tds_dw	tde_type;			/* Type of subsection			*/
  tds_dw	tde_module_index;		/* Starts at 1, 0xffff = no module	*/
  tds_dd	tde_offset;			/* Offset to start of subsection	*/
  tds_dd	tde_size;			/* Number of bytes in subsection	*/
}
tds_directory_entry;

/* These are the defined subsection types */

#define	TDS_SST_MODULE		0x120
#define	TDS_SST_TYPES		0x121
#define	TDS_SST_SYMBOLS		0x124
#define	TDS_SST_ALIGNSYM	0x125
#define	TDS_SST_SRCMODULE	0x127
#define	TDS_SST_GLOBALSYM	0x129
#define	TDS_SST_GLOBALTYPES	0x12b
#define	TDS_SST_NAMES		0x130

/* TDS_SST_MODULE

   This Describes basic information about a module. This section contains
   one module header, followed by tmh_code_segments segment info structures.
    */

typedef struct
{
  tds_dw	tmh_overlay_number;		/* Overlay number for overlayed files	*/
  tds_dw	tmh_library_number;		/* Library number if from a lib file	*/
  tds_dw	tmh_segments;			/* Number of segments			*/
  char		tmh_style[2];			/* Debugging style - only "CV" allowed	*/
  tds_dd	tmh_module_name;		/* Name index of module			*/
  tds_dd	tmh_timestamp;			/* Time stamp of OBJ file		*/
  tds_dd	tmh_reserved[3];		/* Reserved - must be 0.		*/
}
tds_module_header;

typedef struct
{
  tds_dw	tsi_seg_number;			/* Segment number defined by this record*/
  tds_dw	tsi_flags;			/* 0x0000 = data, 0x0001 = code		*/
  tds_dd	tsi_offset;			/* Offset of code from start of segment	*/
  tds_dd	tsi_size;			/* Bytes of code in the segment		*/
}
tds_segment_info;

/* TDS_SST_SRCMODULE

   This section contains line number information for a module. The header
   gives the number of files and segments.

   The header is followed by tlnh_files tds_dd entries giving the offsets
   from the start of this section to the data for each file.

   Next comes tlnh_segments tds_line_number_segment entries giving the
   start and end addresses of this module's code in those segments.

   Next comes tlnh_segments tds_dw entries giving the segment numbers of
   the participating segments.

   The file entries contain a header giving the number of participating
   segments, and the name of the source file. This is followed by
   tlnf_segments tds_dd entries giving the offset to the
   tds_line_number_address table for each segment, and tlnf_segments
   tds_line_number_segment entries giving the range of code in the
   segment that came from the file. This is followed by a single DWORD
   giving the number of bytes in the source file name (a debugger would
   almost never bother to read this).

   The tds_line_number_address header is followed by tlna_lines tds_dd
   entries giving the offsets within the segments for the start of the
   code for each of the lines, and tlna_lines tds_dw entries giving the
   corresponding line number. NOTE - This means that TDS tables cannot
   record line numbers for files with more than 65535 lines.

   A line number of 'zero' indicates the end of the code range of the last
   line in the file. */

typedef struct
{
  tds_dw	tlnh_files;			/* Number of source files in module	*/
  tds_dw	tlnh_segments;			/* Number of segments using this module	*/
}
tds_line_number_header;

typedef struct
{
  tds_dd	tlns_start;			/* Start address of module's code in seg*/
  tds_dd	tlns_end;			/* End address of module's code in seg	*/
}
tds_line_number_segment;

typedef struct
{
  tds_dw	tlnf_segments;			/* The number of segments for this file	*/
  tds_dd	ALIGNED1 (tlnf_name);		/* The name of this file		*/
}
tds_line_number_file;

typedef	struct
{
  tds_dw	tlna_segment;			/* The segment number			*/
  tds_dw	tlna_lines;			/* The number of lines recorded		*/
}
tds_line_number_address;

/* The builtin types use this bitfield */

typedef struct
  {
    tds_dw	tbt_size : 3;			/* Size					*/
    tds_dw	tbt_reserved1 : 1;		/* Must be zero				*/
    tds_dw	tbt_type : 4;			/* Basic datatype			*/
    tds_dw	tbt_mode : 3;			/* Access mode				*/
    tds_dw	tbt_reserved2 : 5;		/* Must be zero				*/
  }
tds_builtin_type;

/* Builting type constants */

#define	TDS_BIT_SPECIAL		0x00		/* Special - Pascal types		*/
#define	TDS_BIT_SIGNED		0x01		/* Signed integer			*/
#define	TDS_BIT_UNSIGNED	0x02		/* Unsigned integer			*/
#define	TDS_BIT_BOOL		0x03		/* Boolean				*/
#define	TDS_BIT_REAL		0x04		/* Real					*/
#define	TDS_BIT_COMPLEX		0x05		/* (-1)**(1/2)				*/
#define	TDS_BIT_SPECIAL2	0x06		/* Special 2 - Pascal			*/
#define	TDS_BIT_REALLYINT	0x07		/* No really, it's an integer.		*/

#define	TDS_SP1_NONE		0x00		/* No type				*/
#define	TDS_SP1_SYMBOL		0x01		/* Absolute symbol			*/
#define	TDS_SP1_SEGMENT		0x02		/* Segment				*/
#define	TDS_SP1_VOID		0x03		/* Void					*/
#define	TDS_SP1_BCURRENCY	0x04		/* BASIC (!) 8 Byte currency		*/
#define	TDS_SP1_BNSTRING	0x05		/* Near BASIC string			*/
#define	TDS_SP1_BFSTRING	0x06		/* Far BASIC string			*/
#define	TDS_SP1_UNSTRANSLATED	0x07		/* Untranslated from older debug format	*/

#define	TDS_INT_INT8		0x00		/* 8 bit integer			*/
#define	TDS_INT_INT16		0x01		/* 16 bit integer			*/
#define	TDS_INT_INT32		0x02		/* 32 bit integer			*/
#define	TDS_INT_INT64		0x02		/* 64 bit integer			*/

/* Note - for complex numbers, the number of bits refers to the number of bits in each of
   the real and imaginary components - thus a COMPLEX128 takes up 256 bits, or 32 bytes */
#define	TDS_CR_32		0x00		/* 32 bit real/complex			*/
#define	TDS_CR_64		0x01		/* 64 bit real/complex			*/
#define	TDS_CR_80		0x02		/* 80 bit real/complex			*/
#define	TDS_CR_128		0x03		/* 128 bit real/complex			*/
#define	TDS_CR_48		0x04		/* 48 bit real/complex			*/

#define	TDS_SP2_BIT		0x00		/* Bit					*/
#define	TDS_SP2_PCHAR		0x01		/* Pascal character			*/

#define	TDS_RI_CHAR		0x00		/* Character				*/
#define	TDS_RI_WCHAR		0x01		/* Wide character			*/
#define	TDS_RI_SHORT		0x02		/* 16 bit				*/
#define	TDS_RI_USHORT		0x04		/* 16 bit, unsigned			*/
#define	TDS_RI_LONG		0x05		/* 32 bit				*/
#define	TDS_RI_ULONG		0x06		/* 32 bit, unsigned			*/
#define	TDS_RI_INT64		0x07		/* 64 bit				*/
#define	TDS_RI_UINT64		0x08		/* Unsigned, 64 bit			*/

/* Alternatively, just use the following numbers, which cover all the base types that
   can actually be found in any executable */

#define TDS_T_NOTYPE		0x0000		/* Uncharacterized type (no type)	*/
#define TDS_T_ABS		0x0001		/* Absolute symbol			*/
#define TDS_T_SEGMENT		0x0002		/* Segment type				*/
#define TDS_T_VOID		0x0003		/* Void					*/
#define TDS_T_PVOID		0x0103		/* Near pointer to void			*/
#define TDS_T_PFVOID		0x0203		/* Far pointer to void			*/
#define TDS_T_PHVOID		0x0303		/* Huge					*/
#define	TDS_T_32PVOID		0x0403		/* 16:32 near pointer to void		*/
#define TDS_T_CURRENCY		0x0004		/* Basic 8 byte currency value		*/
#define TDS_T_NBASICSTR		0x0005		/* Near Basic string			*/
#define TDS_T_FBASICSTR		0x0006		/* Far Basic string			*/
#define TDS_T_NOTTRANS		0x0007		/* Untranslated type record from CV 3.x format */
#define TDS_T_BIT		0x0060		/* Bit					*/
#define TDS_T_PASCHAR		0x0061		/* Pascal CHAR				*/
#define TDS_T_CHAR		0x0010		/* 8-bit signed				*/
#define TDS_T_UCHAR		0x0020		/* 8-bit unsigned			*/
#define TDS_T_PCHAR		0x0110		/* Near pointer to 8-bit signed		*/
#define TDS_T_PUCHAR		0x0120		/* Near pointer to 8-bit unsigned	*/
#define TDS_T_PFCHAR		0x0210		/* Far pointer to 8-bit signed		*/
#define TDS_T_PFUCHAR		0x0220		/* Far pointer to 8-bit unsigned	*/
#define TDS_T_PHCHAR		0x0310		/* Huge pointer to 8-bit signed		*/
#define TDS_T_PHUCHAR		0x0320		/* Huge pointer to 8-bit unsigned	*/
#define TDS_T_32PCHAR		0x0410		/* 16:32 near pointer to 8-bit signed	*/
#define TDS_T_32PUCHAR		0x0420		/* 16:32 near pointer to 8-bit unsigned	*/
#define TDS_T_32PFCHAR		0x0510		/* 16:32 far pointer to 8-bit signed	*/
#define TDS_T_32PFUCHAR		0x0520		/* 16:32 far pointer to 8-bit unsigned	*/
#define TDS_T_RCHAR		0x0070		/* Real char				*/
#define TDS_T_PRCHAR		0x0170		/* Near pointer to a real char		*/
#define TDS_T_PFRCHAR		0x0270		/* Far pointer to a real char		*/
#define TDS_T_PHRCHAR		0x0370		/* Huge pointer to a real char		*/
#define TDS_T_32PRCHAR		0x0470		/* 16:32 near pointer to a real char	*/
#define TDS_T_32PFRCHAR		0x0570		/* 16:32 far pointer to a real char	*/
#define TDS_T_WCHAR		0x0071		/* wide char				*/
#define TDS_T_PWCHAR		0x0171		/* Near pointer to a wide char		*/
#define TDS_T_PFWCHAR		0x0271		/* Far pointer to a wide char		*/
#define TDS_T_PHWCHAR		0x0371		/* Huge pointer to a wide char		*/
#define TDS_T_32PWCHAR		0x0471		/* 16:32 near pointer to a wide char	*/
#define TDS_T_32PFWCHAR		0x0571		/* 16:32 far pointer to a wide char	*/
#define TDS_T_INT2		0x0072		/* really 16-bit signed int		*/
#define TDS_T_UINT2		0x0073		/* really 16-bit unsigned int		*/
#define TDS_T_PINT2		0x0172		/* Near pointer to 16-bit signed int	*/
#define TDS_T_PUINT2		0x0173		/* Near pointer to 16-bit unsigned int	*/
#define TDS_T_PFINT2		0x0272		/* Far pointer to 16-bit signed int	*/
#define TDS_T_PFUINT2		0x0273		/* Far pointer to 16-bit unsigned int	*/
#define TDS_T_PHINT2		0x0372		/* Huge pointer to 16-bit signed int	*/
#define TDS_T_PHUINT2		0x0373		/* Huge pointer to 16-bit unsigned int	*/
#define TDS_T_32PINT2		0x0472		/* 16:32 near pointer to 16-bit signed int */
#define TDS_T_32PUINT2		0x0473		/* 16:32 near pointer to 16-bit unsigned int */
#define TDS_T_32PFINT2		0x0572		/* 16:32 far pointer to 16-bit signed int */
#define TDS_T_32PFUINT2		0x0573		/* 16:32 far pointer to 16-bit		*/
#define TDS_T_SHORT		0x0011		/* 16-bit signed			*/
#define TDS_T_USHORT		0x0021		/* 16-bit unsigned			*/
#define TDS_T_PSHORT		0x0111		/* Near pointer to 16-bit signed	*/
#define TDS_T_PUSHORT		0x0121		/* Near pointer to 16-bit unsigned	*/
#define TDS_T_PFSHORT		0x0211		/* Far pointer to 16-bit signed		*/
#define TDS_T_PFUSHORT		0x0221		/* Far pointer to 16-bit unsigned	*/
#define TDS_T_PHSHORT		0x0311		/* Huge pointer to 16-bit signed	*/
#define TDS_T_PHUSHORT		0x0321		/* Huge pointer to 16-bit unsigned	*/
#define TDS_T_32PSHORT		0x0411		/* 16:32 near pointer to 16-bit signed	*/
#define TDS_T_32PUSHORT		0x0421		/* 16:32 near pointer to 16-bit unsigned */
#define TDS_T_32PFSHORT		0x0511		/* 16:32 far pointer to 16-bit signed	*/
#define TDS_T_32PFUSHORT	0x0521		/* 16:32 far pointer to 16-bit unsigned	*/
#define TDS_T_INT4		0x0074		/* really 32-bit signed int		*/
#define TDS_T_UINT4		0x0075		/* really 32-bit unsigned int		*/
#define TDS_T_PINT4		0x0174		/* Near pointer to 32-bit signed int	*/
#define TDS_T_PUINT4		0x0175		/* Near pointer to 32-bit unsigned int	*/
#define TDS_T_PFINT4		0x0274		/* Far pointer to 32-bit signed int	*/
#define TDS_T_PFUINT4		0x0275		/* Far pointer to 32-bit unsigned int	*/
#define TDS_T_PHINT4		0x0374		/* Huge pointer to 32-bit signed int	*/
#define TDS_T_PHUINT4		0x0375		/* Huge pointer to 32-bit unsigned int	*/
#define TDS_T_32PINT4		0x0474		/* 16:32 near pointer to 32-bit signed int */
#define TDS_T_32PUINT4		0x0475		/* 16:32 near pointer to 32-bit unsigned int */
#define TDS_T_32PFINT4		0x0574		/* 16:32 far pointer to 32-bit signed int */
#define TDS_T_32PFUINT4		0x0575		/* 16:32 far pointer to 32-bit		*/
#define TDS_T_LONG		0x0012		/* 32-bit signed			*/
#define TDS_T_ULONG		0x0022		/* 32-bit unsigned			*/
#define TDS_T_PLONG		0x0112		/* Near pointer to 32-bit signed	*/
#define TDS_T_PULONG		0x0122		/* Near pointer to 32-bit unsigned	*/
#define TDS_T_PFLONG		0x0212		/* Far pointer to 32-bit signed		*/
#define TDS_T_PFULONG		0x0222		/* Far pointer to 32-bit unsigned	*/
#define TDS_T_PHLONG		0x0312		/* Huge pointer to 32-bit signed	*/
#define TDS_T_PHULONG		0x0322		/* Huge pointer to 32-bit unsigned	*/
#define TDS_T_32PLONG		0x0412		/* 16:32 near pointer to 32-bit signed	*/
#define TDS_T_32PULONG		0x0422		/* 16:32 near pointer to 32-bit unsigned */
#define TDS_T_32PFLONG		0x0512		/* 16:32 far pointer to 32-bit signed	*/
#define TDS_T_32PFULONG		0x0522		/* 16:32 far pointer to 32-bit unsigned	*/
#define TDS_T_INT8		0x0076		/* 64-bit signed int			*/
#define TDS_T_UINT8		0x0077		/* 64-bit unsigned int			*/
#define TDS_T_PINT8		0x0176		/* Near pointer to 64-bit signed int	*/
#define TDS_T_PUINT8		0x0177		/* Near pointer to 64-bit unsigned int	*/
#define TDS_T_PFINT8		0x0276		/* Far pointer to 64-bit signed int	*/
#define TDS_T_PFUINT8		0x0277		/* Far pointer to 64-bit unsigned int	*/
#define TDS_T_PHINT8		0x0376		/* Huge pointer to 64-bit signed int	*/
#define TDS_T_PHUINT8		0x0377		/* Huge pointer to 64-bit unsigned int	*/
#define TDS_T_32PINT8		0x0476		/* 16:32 near pointer to 64-bit signed int */
#define TDS_T_32PUINT8		0x0477		/* 16:32 near pointer to 64-bit unsigned int */
#define TDS_T_32PFINT8		0x0576		/* 16:32 far pointer to 64-bit signed int */
#define TDS_T_32PFUINT8		0x0577		/* 16:32 far pointer to 64-bit unsigned int */
#define TDS_T_QUAD		0x0013		/* 64-bit signed			*/
#define TDS_T_UQUAD		0x0023		/* 64-bit unsigned			*/
#define TDS_T_PQUAD		0x0113		/* Near pointer to 64-bit signed	*/
#define TDS_T_PUQUAD		0x0123		/* Near pointer to 64-bit unsigned	*/
#define TDS_T_PFQUAD		0x0213		/* Far pointer to 64-bit signed		*/
#define TDS_T_PFUQUAD		0x0223		/* Far pointer to 64-bit unsigned	*/
#define TDS_T_PHQUAD		0x0313		/* Huge pointer to 64-bit signed	*/
#define TDS_T_PHUQUAD		0x0323		/* Huge pointer to 64-bit unsigned	*/
#define TDS_T_32PQUAD		0x0413		/* 16:32 near pointer to 64-bit signed	*/
#define TDS_T_32PUQUAD		0x0423		/* 16:32 near pointer to 64-bit unsigned */
#define TDS_T_32PFQUAD		0x0513		/* 16:32 far pointer to 64-bit signed	*/
#define TDS_T_32PFUQUAD		0x0523		/* 16:32 far pointer to 64-bit unsigned	*/
#define TDS_T_REAL32		0x0040		/* 32-bit real				*/
#define TDS_T_PREAL32		0x0140		/* Near pointer to 32-bit real		*/
#define TDS_T_PFREAL32		0x0240		/* Far pointer to 32-bit real		*/
#define TDS_T_PHREAL32		0x0340		/* Huge pointer to 32-bit real		*/
#define TDS_T_32PREAL32		0x0440		/* 16:32 near pointer to 32-bit real	*/
#define TDS_T_32PFREAL32	0x0540		/* 16:32 far pointer to 32-bit real	*/
#define TDS_T_REAL48		0x0044		/* 48-bit real				*/
#define TDS_T_PREAL48		0x0144		/* Near pointer to 48-bit real		*/
#define TDS_T_PFREAL48		0x0244		/* Far pointer to 48-bit real		*/
#define TDS_T_PHREAL48		0x0344		/* Huge pointer to 48-bit real		*/
#define TDS_T_32PREAL48		0x0444		/* 16:32 near pointer to 48-bit real	*/
#define TDS_T_32PFREAL48	0x0544		/* 16:32 far pointer to 48-bit real	*/
#define TDS_T_REAL64		0x0041		/* 64-bit real				*/
#define TDS_T_PREAL64		0x0141		/* Near pointer to 64-bit real		*/
#define TDS_T_PFREAL64		0x0241		/* Far pointer to 64-bit real		*/
#define TDS_T_PHREAL64		0x0341		/* Huge pointer to 64-bit real		*/
#define TDS_T_32PREAL64		0x0441		/* 16:32 near pointer to 64-bit real	*/
#define TDS_T_32PFREAL64	0x0541		/* 16:32 far pointer to 64-bit real	*/
#define TDS_T_REAL80		0x0042		/* 80-bit real				*/
#define TDS_T_PREAL80		0x0142		/* Near pointer to 80-bit real		*/
#define TDS_T_PFREAL80		0x0242		/* Far pointer to 80-bit real		*/
#define TDS_T_PHREAL80		0x0342		/* Huge pointer to 80-bit real		*/
#define TDS_T_32PREAL80		0x0442		/* 16:32 near pointer to 80-bit real	*/
#define TDS_T_32PFREAL80	0x0542		/* 16:32 far pointer to 80-bit real	*/
#define TDS_T_REAL128		0x0043		/* 128-bit real				*/
#define TDS_T_PREAL128		0x0143		/* Near pointer to 128-bit real		*/
#define TDS_T_PFREAL128		0x0243		/* Far pointer to 128-bit real		*/
#define TDS_T_PHREAL128		0x0343		/* Huge pointer to 128-bit real		*/
#define TDS_T_32PREAL128	0x0443		/* 16:32 near pointer to 128-bit real	*/
#define TDS_T_32PFREAL128	0x0543		/* 16:32 far pointer to 128-bit real	*/
#define TDS_T_CPLX32		0x0050		/* 32-bit complex			*/
#define TDS_T_PCPLX32		0x0150		/* Near pointer to 32-bit complex	*/
#define TDS_T_PFCPLX32		0x0250		/* Far pointer to 32-bit complex	*/
#define TDS_T_PHCPLX32		0x0350		/* Huge pointer to 32-bit complex	*/
#define TDS_T_32PCPLX32		0x0450		/* 16:32 near pointer to 32-bit complex	*/
#define TDS_T_32PFCPLX32	0x0550		/* 16:32 far pointer to 32-bit complex	*/
#define TDS_T_CPLX64		0x0051		/* 64-bit complex			*/
#define TDS_T_PCPLX64		0x0151		/* Near pointer to 64-bit complex	*/
#define TDS_T_PFCPLX64		0x0251		/* Far pointer to 64-bit complex	*/
#define TDS_T_PHCPLX64		0x0351		/* Huge pointer to 64-bit complex	*/
#define TDS_T_32PCPLX64		0x0451		/* 16:32 near pointer to 64-bit complex	*/
#define TDS_T_32PFCPLX64	0x0551		/* 16:32 far pointer to 64-bit complex	*/
#define TDS_T_CPLX80		0x0052		/* 80-bit complex			*/
#define TDS_T_PCPLX80		0x0152		/* Near pointer to 80-bit complex	*/
#define TDS_T_PFCPLX80		0x0252		/* Far pointer to 80-bit complex	*/
#define TDS_T_PHCPLX80		0x0352		/* Huge pointer to 80-bit complex	*/
#define TDS_T_32PCPLX80		0x0452		/* 16:32 near pointer to 80-bit complex	*/
#define TDS_T_32PFCPLX80	0x0552		/* 16:32 far pointer to 80-bit complex	*/
#define TDS_T_CPLX128		0x0053		/* 128-bit complex			*/
#define TDS_T_PCPLX128		0x0153		/* Near pointer to 128-bit complex	*/
#define TDS_T_PFCPLX128		0x0253		/* Far pointer to 128-bit complex	*/
#define TDS_T_PHCPLX128		0x0353		/* Huge pointer to 128-bit complex	*/
#define TDS_T_32PCPLX128	0x0453		/* 16:32 near pointer to 128-bit complex */
#define TDS_T_32PFCPLX128	0x0553		/* 16:32 far pointer to 128-bit complex	*/
#define TDS_T_BOOL08		0x0030		/* 8-bit boolean			*/
#define TDS_T_BOOL16		0x0031		/* 16-bit boolean			*/
#define TDS_T_BOOL32		0x0032		/* 32-bit boolean			*/
#define TDS_T_PBOOL08		0x0130		/* Near pointer to 8-bit boolean	*/
#define TDS_T_PBOOL16		0x0131		/* Near pointer to 16-bit boolean	*/
#define TDS_T_PBOOL32		0x0132		/* Near pointer to 32-bit boolean	*/
#define TDS_T_PFBOOL08		0x0230		/* Far pointer to 8-bit boolean		*/
#define TDS_T_PFBOOL16		0x0231		/* Far pointer to 16-bit boolean	*/
#define TDS_T_PFBOOL32		0x0232		/* Far pointer to 32-bit boolean	*/
#define TDS_T_PHBOOL08		0x0330		/* Huge pointer to 8-bit boolean	*/
#define TDS_T_PHBOOL16		0x0331		/* Huge pointer to 16-bit boolean	*/
#define TDS_T_PHBOOL32		0x0332		/* Huge pointer to 32-bit boolean	*/
#define TDS_T_32PBOOL08		0x0430		/* 16:32 near pointer to 8-bit boolean	*/
#define TDS_T_32PFBOOL08	0x0530		/* 16:32 far pointer to 8-bit boolean	*/
#define TDS_T_32PBOOL16		0x0431		/* 16:32 near pointer to 16-bit boolean	*/
#define TDS_T_32PFBOOL16	0x0531		/* 16:32 far pointer to 16-bit boolean	*/
#define TDS_T_32PBOOL32		0x0432		/* 16:32 near pointer to 32-bit boolean	*/
#define TDS_T_32PFBOOL32	0x0532		/* 16:32 far pointer to 32-bit boolean	*/

/* Leaf structures for type strings */

#define	TDS_LF_MODIFIER		0x0001		/* Const or volatile			*/
#define	TDS_LF_POINTER		0x0002		/* Pointer				*/
#define	TDS_LF_ARRAY		0x0003		/* Array				*/
#define	TDS_LF_CLASS		0x0004		/* Class				*/
#define	TDS_LF_STRUCTURE	0x0005		/* Structure				*/
#define	TDS_LF_UNION		0x0006		/* Union				*/
#define	TDS_LF_ENUM		0x0007		/* Enumeration				*/
#define	TDS_LF_PROCEDURE	0x0008		/* Procedure				*/
#define	TDS_LF_MFUNCTION	0x0009		/* Member Function			*/
#define	TDS_LF_VTSHAPE		0x000a		/* Virtual function shape table		*/
#define	TDS_LF_COBOL0		0x000b		/* We'll ignore this one - Borland does	*/
#define	TDS_LF_COBOL1		0x000c		/* Same here				*/
#define	TDS_LF_BARRAY		0x000d		/* Same here				*/
#define	TDS_LF_LABEL		0x000e		/* Typeless assembler label		*/
#define	TDS_LF_NULL		0x000f		/* No data				*/
#define	TDS_LF_NOTTRANS		0x0010		/* Untranslatable data type		*/
#define	TDS_LF_DIMARRAY		0x0011		/* Multiply dimensioned array		*/
#define	TDS_LF_VFTPATH		0x0012		/* Virtual function table path		*/

#define	TDS_LF_SKIP		0x0200		/* Reserved space by incremental compiler*/
#define	TDS_LF_ARGLIST		0x0201		/* Argument list			*/
#define	TDS_LF_DEFARG		0x0202		/* Default argument			*/
#define	TDS_LF_LIST		0x0203		/* Arbitrary list			*/
#define	TDS_LF_FIELDLIST	0x0204		/* Field List				*/
#define	TDS_LF_DERIVED		0x0205		/* List of derived classes		*/
#define	TDS_LF_BITFIELD		0x0206		/* Bitfields				*/
#define	TDS_LF_METHODLIST	0x0207		/* Method List				*/
#define	TDS_LF_DIMCONU		0x0208		/* Dimensioned array const upper bound	*/
#define	TDS_LF_DIMCONLU		0x0209		/* Dimensioned array const upper & lower*/
#define	TDS_LF_DIMVARU		0x020a		/* Dimensioned array variable upper	*/
#define	TDS_LF_DIMVARLU		0x020b		/* Dimensioned array var upper & lower	*/
#define	TDS_LF_REFSYM		0x020c		/* Referenced symbol - forbidden	*/

#define	TDS_LF_BCLASS		0x0400		/* Real base class			*/
#define	TDS_LF_VBCLASS		0x0401		/* Virtual base class			*/
#define	TDS_LF_IVBCLASS		0x0402		/* Inherited virtual base class		*/
#define	TDS_LF_ENUMERATE	0x0403		/* Enumeration name and value		*/
#define	TDS_LF_FRIENDFCN	0x0404		/* Friend function			*/
#define	TDS_LF_INDEX		0x0405		/* Index to another type containing rest of list */
#define	TDS_LF_MEMBER		0x0406		/* Data member				*/
#define	TDS_LF_STMEMBER		0x0407		/* Static data member			*/
#define	TDS_LF_METHOD		0x0408		/* Member function			*/
#define	TDS_LF_NESTTYPE		0x0409		/* Nested type (inside a class)		*/
#define	TDS_LF_VFUNCTAB		0x040a		/* Virtual function table		*/
#define	TDS_LF_FRIENDCLS	0x040b		/* Friend class				*/

#define	TDS_LF_NUMBER		0x8000		/* Start of numeric leaves		*/
#define	TDS_LF_CHAR		0x8000		/* Character				*/
#define	TDS_LF_SHORT		0x8001		/* Short				*/
#define	TDS_LF_USHORT		0x8002		/* Unsigned short			*/
#define	TDS_LF_LONG		0x8003		/* Long					*/
#define	TDS_LF_ULONG		0x8004		/* Unsigned long			*/
#define	TDS_LF_REAL32		0x8005		/* Float				*/
#define	TDS_LF_REAL64		0x8006		/* Double				*/
#define	TDS_LF_REAL80		0x8007		/* Long Double				*/
#define	TDS_LF_REAL128		0x8008		/* Huh?					*/
#define	TDS_LF_QUADWORD		0x8009		/* _int64				*/
#define	TDS_LF_UQUADWORD	0x800a		/* unsigned _int64			*/
#define	TDS_LF_REAL48		0x800b		/* Huh?					*/

#define	TDS_LF_PAD0		0xf0		/* Pad zero bytes?			*/
#define	TDS_LF_PAD1		0xf1
#define	TDS_LF_PAD2		0xf2
#define	TDS_LF_PAD3		0xf3
#define	TDS_LF_PAD4		0xf4
#define	TDS_LF_PAD5		0xf5
#define	TDS_LF_PAD6		0xf6
#define	TDS_LF_PAD7		0xf7
#define	TDS_LF_PAD8		0xf8
#define	TDS_LF_PAD9		0xf9
#define	TDS_LF_PAD10		0xfa
#define	TDS_LF_PAD11		0xfb
#define	TDS_LF_PAD12		0xfc
#define	TDS_LF_PAD13		0xfd
#define	TDS_LF_PAD14		0xfe
#define	TDS_LF_PAD15		0xff

/* Use the following to get the size of a structure without its
   trailing numeric leaf */

#define	TDS_NL_SIZE(strct, mem) ((sizeof (strct) - sizeof ((strct).mem))
#define	TDS_IS_PADDING(x)	(((x) & 0xf0) == 0xf0)
#define	TDS_PADDING_SIZE(x)	((x) & 0xf)

/* Numeric value leaves */

typedef struct
  {
    tds_dw	tnc_leaf;			/* Leaf key				*/
    tds_db	tnc_value;			/* Leaf value				*/
  }
tds_numleaf_char;

typedef struct
  {
    tds_dw	tns_leaf;			/* Leaf key				*/
    tds_dw	tns_value;			/* Leaf value				*/	
  }
tds_numleaf_short; /* Also use for ushort */

typedef struct
  {
   tds_dw	tnl_leaf;			/* Leaf key				*/
   tds_dd	ALIGNED1(tnl_value);		/* Leaf value				*/
  }
tds_numleaf_long; /* Also use for ulong */

typedef struct
  {
    tds_dw	tnr_leaf;			/* Leaf key				*/
    float	ALIGNED1(tnr_value);		/* Leaf value				*/
  }
tds_numleaf_real;

typedef struct
  {
    tds_dw	tnd_leaf;			/* Leaf key				*/
    double	ALIGNED1(tnd_value);		/* Leaf value				*/
  }
tds_numleaf_double;

typedef struct
  {
    tds_dw	tne_leaf;			/* Leaf key				*/
    long double	ALIGNED1(tne_value);		/* Leaf value				*/
  }
tds_numleaf_ext;

typedef struct
  {
    tds_dw	tnq_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tnq_low);		/* Low part of value			*/
    tds_dd	ALIGNED1(tnq_high);		/* High part of value			*/
  }
tds_numleaf_quad;

typedef union
  {
    tds_dw			tn_leaf;
    tds_numleaf_char		tn_char;
    tds_numleaf_short		tn_short;
    tds_numleaf_long		tn_long;
    tds_numleaf_real		tn_real;
    tds_numleaf_double		tn_double;
    tds_numleaf_ext		tn_ext;
    tds_numleaf_quad		tn_quad;
  }
tds_numleaf;

/* Use this to declare space for 2 or more numeric leaves at the end of a structure */
#define	TDS_NLSPACE(x, n) char x[(3 + sizeof (tds_numleaf)) * n]

typedef struct
{
  tds_dw	tlp_constructor	: 1;		/* Has a constructor			*/
  tds_dw	tlp_operators	: 1;		/* Has overloaded operators		*/
  tds_dw	tlp_isnested	: 1;		/* Is nested				*/
  tds_dw	tlp_cnested	: 1;		/* Has nested classes			*/
  tds_dw	tlp_opassign	: 1;		/* Has overloaded assignment		*/
  tds_dw	tlp_opcast	: 1;		/* Has overloaded casts			*/
  tds_dw	tlp_fwdref	: 1;		/* This is a forward reference		*/
  tds_dw	tlp_dtor	: 1;		/* Has a destructor			*/
  tds_dw	tlp_reserved	: 7;		/* Shouldn't this be 8?			*/
}
tds_lf_class_property;

typedef struct
{
  tds_dw	tlm_leaf;			/* Leaf key				*/
  tds_dw	tlm_const	: 1;		/* Constant				*/
  tds_dw	tlm_volatile	: 1;		/* Volatile				*/
  tds_dw	tlm_reserved	: 14;		/* Must be zero				*/
  tds_dd	ALIGNED1(tlm_type);		/* Index of modified type		*/
}
tds_lf_modifier;

#define	TDS_PTRTYPE_NEAR	0		/* OFF16				*/
#define	TDS_PTRTYPE_FAR		1		/* SEG16:OFF16				*/
#define	TDS_PTRTYPE_HUGE	2		/* SEG16:OFF16, >64K			*/
#define	TDS_PTRTYPE_SEGBASED	3		/* implied seg:OFF16			*/
#define	TDS_PTRTYPE_VALBASED	4		/* ???					*/
#define	TDS_PTRTYPE_VALSEGBASED	5		/* ???					*/
#define	TDS_PTRTYPE_SYMBASED	6		/* ???					*/
#define	TDS_PTRTYPE_SYMSEGBASED	7		/* ???					*/
#define	TDS_PTRTYPE_TYPEBASED	8		/* ???					*/
#define	TDS_PTRTYPE_SELFBASED	9		/* ???					*/
#define	TDS_PTRTYPE_NEAR32	10		/* OFF32				*/
#define	TDS_PTRTYPE_FAR32	11		/* SEG16:OFF32				*/

#define	TDS_PTRMODE_POINTER	0		/* Pointer				*/
#define	TDS_PTRMODE_REFERENCE	1		/* Reference				*/
#define	TDS_PTRMODE_DMEMPTR	2		/* Pointer to data member		*/
#define	TDS_PTRMODE_METHOD	3		/* Pointer to method			*/

typedef struct
{
  tds_dw	tlp_leaf;			/* Leaf key				*/
  tds_dw	tlp_ptrtype	: 5;		/* TDS_PTRTYPE_xxxx			*/
  tds_dw	tlp_ptrmode	: 3;		/* TDS_PTRMODE_xxxx			*/
  tds_dw	tlp_isflat32	: 1;		/* True if 16:32 pointer		*/
  tds_dw	tlp_volatile	: 1;		/* Volatile pointer			*/
  tds_dw	tlp_const	: 1;		/* Const pointer			*/
  tds_dw	tlp_unused	: 5;		/* Must be zero				*/
  tds_dd	ALIGNED1(tlp_type);		/* Type pointed to			*/
  /* FIXME - Variant data omitted */
}
tds_lf_pointer;

typedef struct
{
  tds_dw	tlp_leaf;			/* Leaf key				*/
  tds_dd	ALIGNED1 (tla_elemtype);	/* Type index of element type		*/
  tds_dd	ALIGNED1 (tla_idxtype);		/* Type index of index type		*/
  tds_dd	ALIGNED1 (tla_name);		/* Name index				*/
  TDS_NLSPACE	(tla_numleaves, 2);		/* Length, then number of elements	*/
}
tds_lf_array;

typedef struct
{
  tds_dw	tlc_leaf;			/* Leaf key				*/
  tds_dw	tlc_count;			/* Number of elements			*/
  tds_dd	tlc_field;			/* Type index of field list		*/
  tds_lf_class_property tlc_property;		/* Properties of the class		*/
  tds_dd	ALIGNED1(tlc_cclass);		/* Type index of containing class	*/
  tds_dd	ALIGNED1(tlc_dlist);		/* Derived classes list	index		*/
  tds_dd	ALIGNED1(tlc_vshape);		/* Index of LF_VSHAPE entry		*/
  tds_dd	ALIGNED1(tlc_name);		/* Class/Struct name			*/
  tds_numleaf	ALIGNED1(tlc_size);		/* Class/Struct size			*/
}
tds_lf_class;

typedef struct
{
  tds_dw	tlu_leaf;			/* Leaf key				*/
  tds_dw	tlu_count;			/* Number of fields			*/
  tds_dd	tlu_field;			/* Type index of field list		*/
  tds_lf_class_property tlu_property;		/* Properties of the union		*/
  tds_dd	ALIGNED1 (tlu_cclass);		/* Type index of containing class	*/
  tds_dd	ALIGNED1 (tlu_name);		/* Name of the union			*/
  tds_numleaf	ALIGNED1 (tlu_size);		/* Size of the union			*/
}
tds_lf_union;

typedef struct
{
  tds_dw	tle_leaf;			/* Leaf key				*/
  tds_dw	tle_count;			/* Number of enumerated values		*/
  tds_dd	tle_type;			/* Type index of base type		*/
  tds_dd	tle_field;			/* Type index of field list		*/
  tds_dd	tle_cclass;			/* Type index of containing class	*/
  tds_dd	tle_name;			/* Name of the enumeration		*/
}
tds_lf_enum;

/* Note that GDB doesn't really need to know about calling conventions */
#define	TDS_CALLCONV_C_NEAR		0	/* C, Near				*/
#define	TDS_CALLCONV_C_FAR		1	/* C, Far				*/
#define	TDS_CALLCONV_PASCAL_NEAR	2	/* Pascal, Near				*/
#define	TDS_CALLCONV_PASCAL_FAR		3	/* Pascal, Far				*/
#define	TDS_CALLCONV_FASTCALL_NEAR	4	/* Fastcall, Near			*/
#define	TDS_CALLCONV_FASTCALL_FAR	5	/* Fastcall, Far			*/
#define	TDS_CALLCONV_STDCALL_NEAR	7	/* Stdcall, Near			*/
#define	TDS_CALLCONV_STDCALL_FAR	8	/* Stdcall, Far				*/
#define	TDS_CALLCONV_SYSCALL_NEAR	9	/* Syscall, Near			*/
#define	TDS_CALLCONV_SYSCALL_FAR	10	/* Syscall, Far				*/
#define	TDS_CALLCONV_THIS		11	/* C++					*/
#define	TDS_CALLCONV_FASTTHIS		0x80	/* this pointer in register		*/

#define	TDS_GET_CALLCONV(x) ((x) & 0x7f)
#define	TDS_IS_FASTTHIS(x) (((x) & TDS_CALLCONV_FASTTHIS) ? 1 : 0)

typedef struct
{
  tds_dw	tlp_leaf;			/* Leaf key				*/
  tds_dd	ALIGNED1 (tlp_rvtype);		/* Type index of returned value		*/
  tds_db	tlp_callconv;			/* Calling convention			*/
  tds_db	tlp_reserved;			/* Must be zero				*/
  tds_dw	tlp_parms;			/* Number of parameters			*/
  tds_dd	ALIGNED1 (tlp_arglist);		/* Type index of argument list		*/
}
tds_lf_procedure;

typedef struct
{
  tds_dw	tlm_leaf;			/* Leaf key				*/
  tds_dd	ALIGNED1 (tlm_rvtype);		/* Type index of returned value		*/
  tds_dd	ALIGNED1 (tlm_class);		/* Type index of containing class	*/
  tds_dd	ALIGNED1 (tlm_this);		/* Type index of "this" pointer		*/
  tds_db	tlm_callconv;			/* Calling convention			*/
  tds_db	tlm_reserved;			/* Must be zero				*/
  tds_dw	tlm_parms;			/* Number of parameters			*/
  tds_dd	ALIGNED1 (tlm_arglist);		/* Type index of argument list		*/
  tds_dd	ALIGNED1 (tlm_thisaddjust);	/* Offset from this pointer used by method */
}
tds_lf_mfunction;

/* The virtual function table shape structure contains a count, which is followed by
   tlv_count 4 bit entries describing the shape of each entry in the virtual table.
   The 4 bit entries are in a DWORD. Unfortunately, the vurtual function shape table
   does not list the full pointer types of its elements, which makes it for all
   practical purposes useless to us (unless we want to start trying to anticipate
   how the compiler populates such tables, which is likely not such a good idea) */

typedef struct
{
  tds_dw	tlv_leaf;			/* Leaf key				*/
  tds_dw	tlv_count;			/* Number of virtual function pointers	*/
}
tds_lf_vtshape;

#define	TDS_VSHAPE_NEAR			0	/* Near VPTR				*/
#define	TDS_VSHAPE_FAR			1	/* Far VPTR				*/
#define	TDS_VSHAPE_THIN			2	/* Thin VPTR (What does this mean?)	*/
#define	TDS_VSHAPE_DISPLACEMENT		3	/* Address displacement to outermost cls*/
#define	TDS_VSHAPE_METACLASS		4	/* Pointer to metaclass descriptor	*/
#define	TDS_VSHAPE_NEAR32		5	/* Near VPTR32				*/
#define	TDS_VSHAPE_FAR32		6	/* Far VPTR32				*/

#define	TDS_LABEL_MODE_NEAR		0	/* Near label				*/
#define	TDS_LABEL_MODE_FAR		4	/* Far label				*/

typedef struct
{
  tds_dw	tll_mode;			/* Mode of label			*/
}
tds_lf_label;

typedef struct
{
  tds_dw	tld_leaf;			/* Leaf key				*/
  tds_dw	tld_utype;			/* Type index of underlying type	*/
  tds_dw	tld_diminfo;			/* Type index of dimension information	*/
  tds_dw	tld_name;			/* Name index of the array		*/
}
tds_lf_dimarray;

/* The tds_ld_vftpath structure actually details the base classes of the current class.
   The structure is followed by tlv_cound tds_dw entries giving the type indeces of the
   base classes. */

typedef struct
{
  tds_dw	tlv_leaf;			/* Leaf key				*/
  tds_dw	tlv_count;			/* Number of base classes		*/
}
tds_lf_vftpath;

typedef union
{
  tds_dw			tl_leaf;
  tds_lf_modifier		tl_modifier;
  tds_lf_pointer		tl_pointer;
  tds_lf_array			tl_array;
  tds_lf_class			tl_class;
  tds_lf_union			tl_union;
  tds_lf_enum			tl_enum;
  tds_lf_procedure		tl_procedure;
  tds_lf_mfunction		tl_mfunction;
  tds_lf_vtshape		tl_vtshape;
  tds_lf_label			tl_label;
  tds_lf_dimarray		tl_dimarray;
  tds_lf_vftpath		tl_vftpath;
}
tds_leaf;

/* Other leaves that stand alone */

typedef struct
  {
    tds_dw	tlb_leaf;			/* Leaf key				*/
    tds_db	tlb_length;			/* Length in bits			*/
    tds_db	tlb_position;			/* Position in bits			*/
    tds_dd	tlb_type;			/* Data type				*/
  }
tds_lf_bitfield;

/* Structure, union and enumeration lists */

#define	TDS_MA_NONE		0		/* No protection			*/
#define	TDS_MA_PRIVATE		1		/* Private				*/
#define	TDS_MA_PROTECTED	2		/* Protected				*/
#define	TDS_MA_PUBLIC		3		/* Public				*/

#define TDS_MPROP_VANILLA	0		/* Vanilla method			*/
#define	TDS_MPROP_VIRTUAL	1		/* Virtual method			*/
#define	TDS_MPROP_STATIC	2		/* Static method			*/
#define	TDS_MPROP_FRIEND	3		/* Friend declaration			*/
#define	TDS_MPROP_INTROVIRT	4		/* Introducing virtual method		*/
#define	TDS_MPROP_PUREVIRT	5		/* Pure virtual method			*/
#define	TDS_MPROP_INTROPURE	6		/* Introducing pure virtual		*/

typedef struct
  {
    tds_dw	tma_access : 2;			/* Access protection			*/
    tds_dw	tma_prop : 3;			/* Method property			*/
    tds_dw	tma_pseudo : 1;			/* Method never instantiated		*/
    tds_dw	tma_noninherit : 1;		/* Cannot be inherited			*/
    tds_dw	tma_noconstruct : 1;		/* Cannot be constructed		*/
    tds_dw	tma_ovlop : 1;			/* Is an overload operator		*/
    tds_dw	tma_cnvop : 1;			/* Is a type conversion operator	*/
    tds_dw	tma_isctor : 1;			/* Is a constructor			*/
    tds_dw	tma_isdtor : 1;			/* Is a destructor			*/
    tds_dw	tma_bltin : 1;			/* Is compiler created method		*/
    tds_dw	tma_isbitfield : 1;		/* Is a bitfield member			*/
    tds_dw	tma_reserved : 2;		/* Must be zero				*/
  }
tds_member_attribute;

typedef struct
  {
    tds_member_attribute tmd_attribs;		/* Attribute				*/
    tds_dd	ALIGNED1(tmd_type);		/* Type record of method		*/
    tds_dd	ALIGNED1(tmd_reserved);		/* Must be zero				*/
  }
tds_methodlist_data;


typedef struct
  {
    tds_dw	tsb_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsb_type);		/* Base class type			*/
    tds_member_attribute tsb_attribs;		/* Base class attributes		*/
  }
tds_structleaf_baseclass;

typedef struct
  {
    tds_dw	tsv_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsv_type);		/* Base class type			*/
    tds_dd	ALIGNED1(tsv_vbptype);		/* Type index of pointer to virtual base */
    tds_member_attribute tsv_attribs;		/* Base class attributes		*/
  }
tds_structleaf_virtualbase;

typedef struct
  {
    tds_dw	tsf_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsf_type);		/* Friend class type			*/
  }
tds_structleaf_friendclass;

typedef struct
  {
    tds_dw	tsf_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsf_type);		/* Friend function type			*/
    tds_dd	ALIGNED1(tsf_name);		/* Friend function name			*/
  }
tds_structleaf_friendfcn;

typedef struct
  {
    tds_dw	tsm_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsm_type);		/* Member type				*/
    tds_member_attribute tsm_attribs;		/* Member attribute			*/
    tds_dd	tsm_name;			/* Member name				*/
    tds_dd	tsm_reserved;			/* Must be zero				*/
  }
tds_structleaf_member;

typedef struct
  {
    tds_dw	tss_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tss_type);		/* Member type				*/
    tds_member_attribute tss_attribs;		/* Member attributes			*/
    tds_dd	tss_name;			/* Member name				*/
    tds_dd	tss_reserved;			/* Must be zero				*/
 }
tds_structleaf_staticmember;

typedef struct
  {
    tds_dw	tsv_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsv_type);		/* Method type				*/
  }
tds_structleaf_vfunctab;

typedef struct
  {
    tds_dw	tsm_leaf;			/* Leaf key				*/
    tds_dw	tsm_count;			/* number of occurences of function	*/
    tds_dd	tsm_mlist;			/* Type index of method list		*/
    tds_dd	tsm_name;			/* Method name				*/
  }
tds_structleaf_method;

typedef struct
  {
    tds_dw	tse_leaf;			/* Leaf key				*/
    tds_member_attribute tse_attribs;		/* Attributes				*/
    tds_dd	tse_name;			/* Name					*/
    tds_dd	tse_reserved;			/* Must be zero				*/
  }
tds_structleaf_enumerate;

typedef struct
  {
    tds_dw	tsn_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsn_type);		/* Type index of nested type		*/
    tds_dd	ALIGNED1(tsn_name);		/* Name of nested type			*/
    tds_dd	ALIGNED1(tsn_reserved);		/* Must be zero				*/
  }
tds_structleaf_nesttype;

typedef struct
  {
    tds_dw	tsi_leaf;			/* Leaf key				*/
    tds_dd	ALIGNED1(tsi_type);		/* Type index of the rest of the list	*/
  }
tds_structleaf_index;

typedef union
  {
    tds_dw			ts_leaf;
    tds_structleaf_baseclass	ts_baseclass;
    tds_structleaf_virtualbase	ts_virtualbase;
    tds_structleaf_friendclass	ts_friendclass;
    tds_structleaf_friendfcn	ts_friendfcn;
    tds_structleaf_member	ts_member;
    tds_structleaf_staticmember	ts_staticmember;
    tds_structleaf_vfunctab	ts_vfunctab;
    tds_structleaf_method	ts_method;
    tds_structleaf_enumerate	ts_enumerate;
    tds_structleaf_nesttype	ts_nesttype;
    tds_structleaf_index	ts_index;
  }
tds_structleaf;

/* The global symbols section contains all symbols that are global
 * to the executable file. */

typedef struct
  {
    tds_dw	tgsh_symhashfunc;		/* Symbol hash function			*/
    tds_dw	tgsh_addrhashfunc;		/* Address hash function		*/
    tds_dd	tgsh_syminfosize;		/* Size of symbol information		*/
    tds_dd	tgsh_symhashsize;		/* Size of symbol hash data		*/
    tds_dd	tgsh_addrhashsize;		/* Size of address hash data		*/
    tds_dd	tgsh_types;			/* Number of user defined types		*/
    tds_dd	tgsh_symbols;			/* Number of global symbols		*/
    tds_dd	tgsh_total;			/* Number of entries			*/
    tds_dd	tgsh_unknown;			/* Don't know what this is		*/
  }
tds_global_symbols_header;

/* The symbol indeces (really just an identifier for the type of record */

#define TDS_S_COMPILE		0x0001		/* Compile flags symbol			*/
#define TDS_S_REGISTER		0x0002		/* Register variable			*/
#define TDS_S_CONST		0x0003		/* Constant symbol			*/
#define TDS_S_UDT		0x0004		/* User-defined Type			*/
#define TDS_S_SSEARCH		0x0005		/* Start search				*/
#define TDS_S_END		0x0006		/* End block, procedure, with, or thunk	*/
#define TDS_S_SKIP		0x0007		/* Skip - Reserve symbol space		*/
#define TDS_S_CVRESERVE		0x0008		/* Reserved for Code View internal use	*/
#define TDS_S_OBJNAME		0x0009		/* Specify name of object file		*/
#define	TDS_S_GPROCREF		0x0020		/* Global procedure			*/
#define	TDS_S_GDATAREF		0x0021		/* Global data				*/
#define	TDS_S_EDATA		0x0022		/* External data			*/
#define	TDS_S_EPROC		0x0023		/* External procedure			*/

#define TDS_S_BPREL16		0x0100		/* BP relative 16:16			*/
#define TDS_S_LDATA16		0x0101		/* Local data 16:16			*/
#define TDS_S_GDATA16		0x0102		/* Global data 16:16			*/
#define TDS_S_PUB16		0x0103		/* Public symbol 16:16			*/
#define TDS_S_LPROC16		0x0104		/* Local procedure start 16:16		*/
#define TDS_S_GPROC16		0x0105		/* Global procedure start 16:16		*/
#define TDS_S_THUNK16		0x0106		/* Thunk start 16:16			*/
#define TDS_S_BLOCK16		0x0107		/* Block start 16:16			*/
#define TDS_S_WITH16		0x0108		/* With start 16:16			*/
#define TDS_S_LABEL16		0x0109		/* Code label 16:16			*/
#define TDS_S_CEXMODEL16	0x010a		/* Change execution model 16:16		*/
#define TDS_S_VFTPATH16		0x010b		/* Virtual function table path descriptor 16:16 */

#define TDS_S_BPREL32		0x0200		/* BP relative 16:32			*/
#define TDS_S_LDATA32		0x0201		/* Local data 16:32			*/
#define TDS_S_GDATA32		0x0202		/* Global data 16:32			*/
#define TDS_S_PUB32		0x0203		/* Public symbol 16:32			*/
#define TDS_S_LPROC32		0x0204		/* Local procedure start 16:32		*/
#define TDS_S_GPROC32		0x0205		/* Global procedure start 16:32		*/
#define TDS_S_THUNK32		0x0206		/* Thunk start 16:32			*/
#define TDS_S_BLOCK32		0x0207		/* Block start 16:32			*/
#define TDS_S_WITH32		0x0208		/* With start 16:32			*/
#define TDS_S_LABEL32		0x0209		/* Label 16:32				*/
#define TDS_S_CEXMODEL32	0x020a		/* Change execution model 16:32		*/
#define TDS_S_VFTPATH32		0x020b		/* Virtual function table path descriptor 16:32 */

/* Standard header */

typedef struct
  {
    tds_dw	tsh_length;			/* Length of record, excluding length	*/
    tds_dw	tsh_symindex;			/* The symbol ID			*/
  }
tds_symbol_header;

/* TDS_S_COMPILE 0x0001  - Compile flags symbol */

#define TDS_C_MACH_8080		0x00		/* Intel 8080				*/
#define TDS_C_MACH_8086		0x01		/* Intel 8086				*/
#define TDS_C_MACH_80286	0x02		/* Intel 80286				*/
#define TDS_C_MACH_80386	0x03		/* Intel 80386				*/
#define TDS_C_MACH_80486	0x04		/* Intel 80486				*/
#define TDS_C_MACH_R4000	0x10		/* MIPS R4000				*/

#define TDS_C_LANG_C		0		/* C					*/
#define TDS_C_LANG_CPLUSPLUS	1		/* C++					*/
#define TDS_C_LANG_FORTRAN	2		/* FORTRAN				*/
#define TDS_C_LANG_ASSEMBLY	3		/* Masm					*/
#define TDS_C_LANG_PASCAL	4		/* Pascal				*/
#define TDS_C_LANG_BASIC	5		/* Basic				*/
#define TDS_C_LANG_COBOL	6		/* COBOL				*/

#define	TDS_C_MODEL_NEAR	0		/* Near memory model			*/
#define	TDS_C_MODEL_FAR		1		/* Far memory model			*/
#define	TDS_C_MODEL_HUGE	2		/* Huge memory model			*/

#define	TDS_C_MATH_HARDWARD	0		/* Hardware FP math			*/
#define	TDS_C_MATH_EMULATOR	1		/* FP Emulator included			*/
#define	TDS_C_MATH_ALTMATH	2		/* Altmath				*/

typedef struct
  {
    tds_dw	tsc_length;			/* Length of record, excluding length	*/
    tds_dw	tsc_symindex;			/* The symbol ID			*/
    tds_dd	tsc_language : 8;		/* The language identifier		*/
    tds_dd	tsc_pcode : 1;			/* Packed code present (?)		*/
    tds_dd	tsc_floatprecision : 2;		/* 1 if -OP was used			*/
    tds_dd	tsc_floatpackage : 2;		/* Floating point math type ID		*/
    tds_dd	tsc_ambientdata : 3;		/* Data memory model			*/
    tds_dd	tsc_ambientcode : 3;		/* Code memory model			*/
    tds_dd	tsc_mode32 : 1;			/* 1 if 32 bit compilation		*/
    tds_dd	tsc_chsign : 1;			/* 1 if default character signed	*/
    tds_dd	tsc_reserved : 3;		/* Must be zero				*/
    tds_dd	tsc_machine : 8;		/* Machine Identifier			*/
    tds_db	tsc_compiler_length;		/* Length of compiler type string	*/
    char	tsc_compiler[1];		/* Compiler type name			*/
  }
tds_symbol_compile;

/* TDS_S_REGISTER 0x0002  - Register variable */

typedef struct
  {
    tds_dw	tsr_length;			/* Length of record, excluding length	*/
    tds_dw	tsr_symindex;			/* The symbol ID			*/
    tds_dd	tsr_type;			/* The data type			*/
    tds_dw	tsr_register;			/* The register ID			*/
    tds_dd	ALIGNED1(tsr_name);		/* The variable name			*/
    tds_dd	ALIGNED1(tsr_reserved);		/* Must be zero				*/
  }
tds_symbol_register;

/* TDS_S_CONST 0x0003  - Constant symbol */

typedef struct
  {
    tds_dw	tsc_length;			/* Length of record, excluding length	*/
    tds_dw	tsc_symindex;			/* The symbol ID			*/
    tds_dd	tsc_type;			/* The data type			*/
    tds_dd	tsc_name;			/* Name					*/
    tds_dd	tsc_reserved2;			/* Must be zero				*/
    tds_numleaf	tsc_value;			/* The value				*/
  }
tds_symbol_const;

/* TDS_S_UDT 0x0004  - User-defined Type */

typedef struct
  {
    tds_dw	tsu_length;			/* Length of record, excluding length	*/
    tds_dw	tsu_symindex;			/* The symbol ID			*/
    tds_dd	tsu_type;			/* The data type			*/
    tds_dw	tsu_istag : 1;			/* True if not a typedef		*/
    tds_dw	tsu_isnest : 1;			/* True if nested			*/
    tds_dw	tsu_reserved1 : 14;		/* Must be zero				*/
    tds_dd	ALIGNED1(tsu_name);		/* Name					*/
    tds_dd	ALIGNED1(tsu_reserved2);	/* Must be zero				*/	
  }
tds_symbol_udt;

/* TDS_S_SSEARCH 0x0005  - Start search */
/* TDS_S_END 0x0006  - End block, procedure, with, or thunk
   This has no data */
/* TDS_S_SKIP 0x0007  - Skip - Reserve symbol space */
/* TDS_S_CVRESERVE 0x0008  - Reserved for Code View internal use */
/* TDS_S_OBJNAME 0x0009  - Specify name of object file */

typedef struct
  {
    tds_dw	tso_length;			/* Length of record, excluding length	*/
    tds_dw	tso_symindex;			/* The symbol ID			*/
    tds_dd	tso_signature;			/* CodeView signature of module		*/
    tds_dd	tso_name;			/* Name index of module name		*/
  }
tds_symbol_objname;

/* TDS_S_GPROCREF 0x0020 - Global procedure */

typedef struct
  {
    tds_dw	tsp_length;			/* Length of record, excluding length	*/
    tds_dw	tsp_symindex;			/* The symbol ID			*/
    tds_dd	tsp_offset;			/* Offset from start of debug to S_GPROCxx */
    tds_dd	tsp_type;			/* Data type				*/
    tds_dd	tsp_name;			/* Name					*/
    tds_dd	tsp_reserved;			/* Must be zero				*/
    tds_dd	tsp_coffset;			/* Code offset				*/
    tds_dw	tsp_segment;			/* Segment				*/
  }
tds_symbol_procref;

/* TDS_S_GDATAREF 0x0021 - Global data */

typedef struct
  {
    tds_dw	tsd_length;			/* Length of record, excluding length	*/
    tds_dw	tsd_symindex;			/* The symbol ID			*/
    tds_dd	tsd_offset;			/* Must be zero				*/
    tds_dd	tsd_type;			/* Data type				*/
    tds_dd	tsd_name;			/* Name					*/
    tds_dd	tsd_reserved;			/* Must be zero				*/
    tds_dd	tsd_doffset;			/* Data offset				*/
    tds_dw	tsd_segment;			/* Segment				*/
  }
tds_symbol_dataref;

/* These items have no value to us. We don't care about symbols external to a given module, and
   the symbols with 16 bit offsets and addresses should never appear in an executable to be
   
   TDS_S_EDATA 0x0022 - External data 
   TDS_S_EPROC 0x0023 - External procedure 

   TDS_S_BPREL16 0x0100  - BP relative 16:16 
   TDS_S_LDATA16 0x0101  - Local data 16:16 
   TDS_S_GDATA16 0x0102  - Global data 16:16 
   TDS_S_PUB16 0x0103  - Public symbol 16:16 
   TDS_S_LPROC16 0x0104  - Local procedure start 16:16 
   TDS_S_GPROC16 0x0105  - Global procedure start 16:16 
   TDS_S_THUNK16 0x0106  - Thunk start 16:16 
   TDS_S_BLOCK16 0x0107  - Block start 16:16 
   TDS_S_WITH16 0x0108  - With start 16:16 
   TDS_S_LABEL16 0x0109  - Code label 16:16 
   TDS_S_CEXMODEL16 0x010a  - Change execution model 16:16 
   TDS_S_VFTPATH16 0x010b  - Virtual function table path descriptor 16:16 */

/* TDS_S_BPREL32 0x0200  - BP relative 16:32 */

typedef struct
  {
    tds_dw	tsb_length;			/* Length of record, excluding length	*/
    tds_dw	tsb_symindex;			/* The symbol ID			*/
    tds_dd	tsb_offset;			/* Offset from BP			*/
    tds_dd	tsb_type;			/* Data type				*/
    tds_dd	tsb_name;			/* Name					*/
  }
tds_symbol_bprel32;

/* TDS_S_LDATA32 0x0201  - Local data 16:32
   TDS_S_GDATA32 0x0202  - Global data 16:32
   TDS_S_PUB32 0x0203  - Public symbol 16:32 */

typedef struct
  {
    tds_dw	tsg_length;			/* Length of record, excluding length	*/
    tds_dw	tsg_symindex;			/* The symbol ID			*/
    tds_dd	tsg_offset;			/* Offset component			*/
    tds_dw	tsg_segment;			/* Segment component			*/
    tds_dw	tsg_reserved;			/* Must be zero				*/
    tds_dd	tsg_type;			/* Type index				*/
    tds_dd	tsg_name;			/* Symbol name				*/
  }
tds_symbol_glob32;

/* TDS_S_LPROC32 0x0204  - Local procedure start 16:32
   TDS_S_GPROC32 0x0205  - Global procedure start 16:32

   The Borland documentation shows a different layout for this structure,
   but from examining the source files the Borland docs are clearly wrong. */

typedef struct
  {
    tds_dw	tsp_length;			/* Length of record, excluding length	*/
    tds_dw	tsp_symindex;			/* The symbol ID			*/
    tds_dd	tsp_pparent;			/* Parent scope				*/
    tds_dd	tsp_pend;			/* End scope ?				*/
    tds_dd	tsp_pnext;			/* Next sibling ?			*/
    tds_dd	tsp_proclength;			/* Length of procedure code		*/
    tds_dd	tsp_debugstart;			/* Position where stack frame established */
    tds_dd	tsp_debugend;			/* Position before stack frame unwound	*/
    tds_dd	tsp_offset;			/* Offset component			*/
    tds_dw	tsp_segment;			/* Segment component			*/
    tds_db	tsp_model;			/* 0 = near, 4 == far (suspected)	*/
    tds_db	tsp_reserved;			/* Must be zero (suspected)		*/
    tds_dd	tsp_proctype;			/* Type index describing arguments and retval */
    tds_dd	tsp_name;			/* Procedure name			*/
  }
tds_symbol_proc32;

/* TDS_S_THUNK32 0x0206  - Thunk start 16:32 */

typedef struct
  {
    tds_dw	tst_length;			/* Length of record, excluding length	*/
    tds_dw	tst_symindex;			/* The symbol ID			*/
    tds_dd	tst_pparent;			/* Parent scope				*/
    tds_dd	tst_pend;			/* End scope ?				*/
    tds_dd	tst_pnext;			/* Next Scope?				*/
    tds_dd	tst_offset;			/* Offset component			*/
    tds_dw	tst_segment;			/* Segment component			*/
    tds_dw	tst_thunklength;		/* Thunk length				*/
    tds_db	tst_ordinal;			/* Type of thunk			*/
    tds_dd	ALIGNED1(tst_name);		/* Name - name of the block		*/
  }
tds_symbol_thunk32;

/* TDS_S_BLOCK32 0x0207  - Block start 16:32 */

typedef struct
  {
    tds_dw	tsb_length;			/* Length of record, excluding length	*/
    tds_dw	tsb_symindex;			/* The symbol ID			*/
    tds_dd	tsb_pparent;			/* Parent scope				*/
    tds_dd	tsb_pend;			/* End scope ?				*/
    tds_dd	tsb_blocklength;		/* Length of block			*/
    tds_dd	tsb_offset;			/* Offset component			*/
    tds_dw	tsb_segment;			/* Segment component			*/
    tds_dd	ALIGNED1(tsb_name);		/* Name - name of the block		*/
  }
tds_symbol_block32;

/* TDS_S_WITH32 0x0208  - With start 16:32 */
/* TDS_S_LABEL32 0x0209  - Label 16:32 */
/* TDS_S_CEXMODEL32 0x020a  - Change execution model 16:32 */
/* TDS_S_VFTPATH32 0x020b  - Virtual function table path descriptor 16:32 */

typedef union
  {
    tds_symbol_header	ts_header;
    tds_symbol_compile	ts_compile;
    tds_symbol_glob32	ts_global32;
    tds_symbol_proc32	ts_procedure32;
    tds_symbol_procref	ts_procref;
    tds_symbol_dataref	ts_dataref;
    tds_symbol_thunk32	ts_thunk32;
    tds_symbol_block32	ts_block32;
    tds_symbol_bprel32	ts_bprel32;
    tds_symbol_register	ts_register;
    tds_symbol_const	ts_const;
    tds_symbol_udt	ts_udt;
    tds_symbol_objname	ts_objname;
  }
tds_symbol;

#define	TDS_REG_AL	1
#define	TDS_REG_CL	2
#define	TDS_REG_DL	3
#define	TDS_REG_BL	4
#define	TDS_REG_AH	5
#define	TDS_REG_CH	6
#define	TDS_REG_DH	7
#define	TDS_REG_BH	8
#define	TDS_REG_AX	9
#define	TDS_REG_CX	10
#define	TDS_REG_DX	11
#define	TDS_REG_BX	12
#define	TDS_REG_SP	13
#define	TDS_REG_BP	14
#define	TDS_REG_SI	15
#define	TDS_REG_DI	16
#define	TDS_REG_EAX	17
#define	TDS_REG_ECX	18
#define	TDS_REG_EDX	19
#define	TDS_REG_EBX	20
#define	TDS_REG_ESP	21
#define	TDS_REG_EBP	22
#define	TDS_REG_ESI	23
#define	TDS_REG_EDI	24
#define	TDS_REG_ES	25
#define	TDS_REG_CS	26
#define	TDS_REG_SS	27
#define	TDS_REG_DS	28
#define	TDS_REG_FS	29
#define	TDS_REG_GS	30
#define	TDS_REG_IP	31
#define	TDS_REG_FLAGS	32
#define	TDS_REG_EIP	33

#define	TDS_REG_PCODE_TEMP	40
#define	TDS_REG_PCODE_TEMPH	41
#define	TDS_REG_PCODE_QUOTE	42

#define	TDS_REG_FP_0		128
#define	TDS_REG_FP_1		129
#define	TDS_REG_FP_2		130
#define	TDS_REG_FP_3		131
#define	TDS_REG_FP_4		132
#define	TDS_REG_FP_5		133
#define	TDS_REG_FP_6		134
#define	TDS_REG_FP_7		135

#define	TDS_REG_FP_CONTROL	136
#define	TDS_REG_FP_STATUS	137
#define	TDS_REG_FP_TAG		138
#define	TDS_REG_FP_FPIP		139
#define	TDS_REG_FP_FPCS		140
#define	TDS_REG_FP_FPDO		141
#define	TDS_REG_FP_FPDS		142
#define	TDS_REG_FP_ISEM		143

#ifdef __BORLANDC__
#pragma -a.
#endif
